package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.EnumUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.unions.ChannelUnion;
import net.dv8tion.jda.api.events.channel.ChannelCreateEvent;
import net.dv8tion.jda.api.events.channel.ChannelDeleteEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ChannelListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public ChannelListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onChannelCreate(@NonNull BatGuild guild, @NonNull ChannelCreateEvent event) {
        logFeature.sendLog(guild, LogType.CHANNEL_CREATE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("%s Channel Created".formatted(EnumUtils.getEnumName(event.getChannel().getType())))
                        .appendLine("Channel: %s".formatted(event.getChannel().getAsMention()), true)
                        .appendLine("Name: %s".formatted(event.getChannel().getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onChannelDelete(@NonNull BatGuild guild, @NonNull ChannelDeleteEvent event) {
        ChannelUnion channel = event.getChannel();
        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("%s Channel Deleted".formatted(EnumUtils.getEnumName(channel.getType())))
                .appendLine("Name: #%s".formatted(channel.getName()), true);
        if (channel.getType().isMessage()) {
            TextChannel textChannel = channel.asTextChannel();
            description.appendLine("Topic: %s".formatted(textChannel.getTopic()), true);
        }
        logFeature.sendLog(guild, LogType.CHANNEL_DELETE, EmbedUtils.errorEmbed().setDescription(description.build()).build());
    }
}
